/*
 * Copyright (C) 2002 2003 2004, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#include <config.h>

#include <math.h>
#include <gtk/gtk.h>
#include "mainwindow.h"
#include "sampleratedialog.h"
#include "chunk.h"
#include "float_box.h"
#include "soxdialog.h"
#include "pipedialog.h"
#include "um.h"
#include "inifile.h"
#include "effectbrowser.h"

static gchar *resample_string = "(SoX) Simulated analog filtration";
static gchar *polyphase_string = "(SoX) Polyphase interpolation";

static void set_method(GtkList *list, GtkWidget *widget, gpointer user_data)
{
     SamplerateDialog *s = SAMPLERATE_DIALOG(user_data);
     s->method = gtk_list_child_position(list,widget);
}

static Chunk *apply_proc(Chunk *chunk, StatusBar *bar, gpointer user_data)
{
     Chunk *c;
     SamplerateDialog *s = SAMPLERATE_DIALOG(user_data);
     gchar buf[128],buf2[32],buf3[32];
     gchar *eff;
     switch (s->method) {
     case 0:
	  return chunk_convert_samplerate(chunk,s->rate->val,
					  INTERPOLATION_NONE,bar);
     case 1:
	  return chunk_convert_samplerate(chunk,s->rate->val,
					  INTERPOLATION_LINEAR,bar);
     case 2:
	  eff = "resample";
	  break;
     case 3:
	  eff = "polyphase";
	  break;
     default:
	  g_assert_not_reached();
	  return NULL;
     }     
     sox_dialog_format_string(buf2,sizeof(buf2),chunk->format.samplerate,
			      chunk->format.sign,chunk->format.samplesize,
			      chunk->format.channels);
     sox_dialog_format_string(buf3,sizeof(buf3),s->rate->val,
			      chunk->format.sign,chunk->format.samplesize,
			      chunk->format.channels);	    
     g_snprintf(buf,sizeof(buf),"sox %s - %s - %s",buf2,buf3,eff);
     c = pipe_dialog_pipe_chunk(chunk,buf,FALSE,bar);
     if (c != NULL) {
	  c->format.samplerate = s->rate->val;
	  ((DataPart *)(c->parts->data))->ds->format.samplerate = s->rate->val;
     }
     return c;
}

static gboolean apply(EffectDialog *ed)
{
     gboolean b;
     Mainwindow *w = EFFECT_BROWSER(ed->eb)->mwl->selected;
     SamplerateDialog *s = SAMPLERATE_DIALOG(ed);
     if (intbox_check(s->rate) || s->rate->val==0) return TRUE;
     if (s->method > 1 && (w->view->chunk->format.samplesize==3 || 
			   w->view->chunk->format.samplesize>4)){
	  user_info("The SoX samplerate conversion only supports 8, 16 and "
		    "32-bit samples");
	  return TRUE;
     }
     inifile_set_guint32("srate_method",s->method);
     b = mainwindow_effect_manual(w,apply_proc,FALSE,ed);
     mainwindow_update_texts();
     return b;
}

static void samplerate_dialog_setup(EffectDialog *ed)
{
     intbox_set(SAMPLERATE_DIALOG(ed)->rate,
		EFFECT_BROWSER(ed->eb)->mwl->format.samplerate);
}

static void samplerate_dialog_class_init(EffectDialogClass *klass)
{
     klass->apply = apply;
     klass->setup = samplerate_dialog_setup;
}

static void samplerate_dialog_init(SamplerateDialog *v)
{
     EffectDialog *ed = EFFECT_DIALOG(v);
     GtkWidget *a,*b,*c;
     GtkRequisition req;
     GList *l = NULL;
     guint u;

     a = gtk_vbox_new ( FALSE, 3 );
     gtk_box_pack_start (GTK_BOX(ed->input_area),a,FALSE,FALSE,0);
     b = gtk_label_new ( "(This changes the sample rate of \n"
			 "the entire file, not just the selection)" );
     gtk_box_pack_start ( GTK_BOX(a),b,FALSE,FALSE,0 );
     b = gtk_hbox_new ( FALSE, 3 );
     gtk_box_pack_start (GTK_BOX(a),b,FALSE,FALSE,0);
     c = gtk_label_new ( "New samplerate: " );
     gtk_box_pack_start (GTK_BOX(b),c,FALSE,FALSE,0);
     c = intbox_new(0);
     v->rate = INTBOX(c);
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     b = gtk_hbox_new( FALSE, 3 );
     gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
     c = gtk_label_new("Method: ");
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     c = gtk_combo_new();
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_size_request(c,&req);
     gtk_widget_set_usize(c,req.width*3/2,req.height);
     l = g_list_append(l,"Sample repeat");
     l = g_list_append(l,"Linear interpolation");
     if (sox_dialog_first_effect() != NULL) {
	  l = g_list_append(l,resample_string);
	  l = g_list_append(l,polyphase_string);
     }
     u = g_list_length(l)>2 ? 2 : 1;
     v->method = inifile_get_guint32("srate_method",u);
     if (v->method >= g_list_length(l)) v->method = u;
     gtk_combo_set_popdown_strings(GTK_COMBO(c),l);
     gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(c)->entry),FALSE);
     gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(c)->entry),
			(gchar *)g_list_nth_data(l,v->method));
     g_list_free(l);
     gtk_signal_connect(GTK_OBJECT(GTK_COMBO(c)->list),"select-child",
			GTK_SIGNAL_FUNC(set_method),v);

     gtk_widget_show_all(a);
}

GtkType samplerate_dialog_get_type(void)
{
     static GtkType id = 0;
     if (!id) {
	  GtkTypeInfo info = {
	       "SamplerateDialog",
	       sizeof(SamplerateDialog),
	       sizeof(SamplerateDialogClass),
	       (GtkClassInitFunc) samplerate_dialog_class_init,
	       (GtkObjectInitFunc) samplerate_dialog_init
	  };
	  id = gtk_type_unique(effect_dialog_get_type(),&info);
     }
     return id;
}
